﻿using BA.API;
using BA.Core.Configuration.AdditionalAttributes;
using BA.Core.Configuration.Application.SubConfigurations;
using BA.Core.Configuration.Form.Controls;
using BA.Core.CustomAttributes;
using BA.Core.Models;
using BA.Core.Tools;
using DevExpress.Web;
using DevExpress.Web.Mvc;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Web;
using System.Web.UI.WebControls;

namespace BA.Core.Configuration.Form.Renderers
{
    /// <summary>
    /// Renderer for the new control
    /// </summary>
    [FormControlRenderer(typeof(ValueAndAliasControl))]
    public class ValueAndAliasDefaultRenderer : ControlDefaultRendererBase
    {
        /// <inheritdoc/>
        public override void EditMode(FormControlRendererParameterModel parameters)
        {
            FormControlRendererUiParameterModel parametersUi = (FormControlRendererUiParameterModel)parameters;
            ValueAndAliasControl valueAndAliasControl = (ValueAndAliasControl)parametersUi.Control;
            FormRenderingContextUi renderingContext = (FormRenderingContextUi)parametersUi.RenderingContext;

            // Feldnamen definieren (Datenmodell)
            String modelFieldName = valueAndAliasControl.OrmFieldName;

            // Feldnamen definieren für erstes Feld (Browser UI)
            String uiFieldName = modelFieldName + "_Value";

            // zu verwendendes Trennzeichen
            char delimiter = valueAndAliasControl.Delimiter.ToCharArray()[0];

            // Inhalt des gespeicherten Feldes holen und für das erste Feld bearbeiten
            String fieldContent = (String)parametersUi.BindData.GetPropertyValue(modelFieldName);
            fieldContent = fieldContent?.Split(delimiter)?[0] ?? "";

            // Erstes Feld erzeugen und als Objekt hinzufügen
            MVCxFormLayoutItem mvcxItem = CreateTextBoxItem(valueAndAliasControl, uiFieldName, renderingContext.FormModel, "Wert", fieldContent);
            parametersUi.MvcxControls.Add(mvcxItem);

            // Inhalt des gespeicherten Feldes holen und für das zweite Feld bearbeiten
            fieldContent = (String)parametersUi.BindData.GetPropertyValue(modelFieldName);
            String[] splt = fieldContent?.Split(delimiter) ?? (new String[] { "", "" });

            if (splt.Length < 2)
                splt = fieldContent?.Split(delimiter) ?? (new String[] { "", "" });

            fieldContent = fieldContent?.Split(delimiter)?[1] ?? "";

            // Feldnamen definieren für zweites Feld (Browser UI)
            uiFieldName = modelFieldName + "_Alias";

            // Zweites Feld erzeugen und als Objekt hinzufügen
            mvcxItem = CreateTextBoxItem(valueAndAliasControl, uiFieldName, renderingContext.FormModel, "Alias", fieldContent);
            parametersUi.MvcxControls.Add(mvcxItem);
        }

        /// <summary>
        /// Methode zum erstellen einer Textbox
        /// </summary>
        /// <param name="valueAndAliasControl"></param>
        /// <param name="fieldName"></param>
        /// <param name="formModel"></param>
        /// <param name="label"></param>
        /// <param name="presetValue"></param>
        /// <returns></returns>
        private MVCxFormLayoutItem CreateTextBoxItem(ValueAndAliasControl valueAndAliasControl,
                String fieldName, DevExFormModel formModel, String label, String presetValue)
        {
            MVCxFormLayoutItem mvcxItem = new MVCxFormLayoutItem();

            // Der Typ des neuen Controls => es ist eine "TextBox".
            mvcxItem.NestedExtensionType = FormLayoutNestedExtensionItemType.TextBox;

            // setzen verschiedener Einstellungen auf das MVCxItem, wie zB der Feldname, der im  
            // Webbrowser verwendet werden soll, die Beschriftung vor dem Feld, der Hilfetext, etc
            mvcxItem.Name = fieldName;
            mvcxItem.FieldName = fieldName;
            //mvcxItem.HelpText = RenderingUtils.GetHelpText(Api.Text.Format(valueAndAliasControl.HelpText));
            mvcxItem.ClientVisible = valueAndAliasControl.Visible;
            mvcxItem.VerticalAlign = FormLayoutVerticalAlign.Top;
            mvcxItem.Caption = label;
            mvcxItem.CaptionSettings.Location = Api.Enum.EnumValueConverter.GetLabelPosition(valueAndAliasControl.LabelPosition);
            mvcxItem.CaptionSettings.VerticalAlign = FormLayoutVerticalAlign.Top;

            // Weitere bestimmte Einstellungen sind im sog. "Settings"-Unterobjekt verfügbar. Das 20:    // betrifft zB die Editierbarkeit des Feldes, bestimmte Javascript-Events im Browser, 21:    // die ausgeführt werden sollen, Abmessungen des Feldes, usw.
            TextBoxSettings settings = (TextBoxSettings)mvcxItem.NestedExtensionSettings;

            // Horizontale Ausrichtung des Controls
            settings.ControlStyle.HorizontalAlign = HorizontalAlign.Left;

            // ist das Feld eventuell für Bearbeitung gesperrt?
            Dictionary<AttributeEnum, Boolean> attributes = RenderingUtils.GetFormAttributesForControl(formModel.AdditionalAttributes, valueAndAliasControl);
            if (attributes.ContainsKey(AttributeEnum.ReadOnly) || valueAndAliasControl.ReadOnly)
                settings.ClientEnabled = false;

            // Breite des Feldes
            settings.Width = new Unit(valueAndAliasControl.Width, Api.Enum.EnumValueConverter.GetValueWidthType(valueAndAliasControl.WidthType));

            // ist es ein Pflichtfeld?
            if (valueAndAliasControl.Required)
                settings.Properties.ValidationSettings.RequiredField.IsRequired = true;

            // Feld kann als fehlerhaft markiert werden, falls die Validierung fehlschlägt
            if (settings.Properties is EditProperties)
            {
                EditProperties props = (EditProperties)settings.Properties;
                ApplicationSubConfigurationForm appFormSettings = Api.Config.ApplicationConfiguration().GetSubConfiguration<ApplicationSubConfigurationForm>();

                props.ValidationSettings.Display = Api.Enum.EnumValueConverter.GetErrorFrameDisplayMode(appFormSettings.ErrorFrameDisplayMode);
                props.ValidationSettings.ErrorTextPosition = Api.Enum.EnumValueConverter.GetErrorTextPosition(appFormSettings.ErrorTextPosition);
                props.ValidationSettings.ErrorDisplayMode = Api.Enum.EnumValueConverter.GetErrorDisplayMode(appFormSettings.ErrorDisplayMode);
            }

            // Wenn das Feld aktiv wird, müssen eventuell die Menüs des HTML-Controls ausgeblendet 51:    // werden (das geht technisch leider nicht, wenn man das HTML-Control verlässt, daher 52:    // müssen sich alle anderen Controls darum kümmern)
            settings.Properties.ClientSideEvents.GotFocus = "BA.Ui.RibbonUtils.HideHtmlEditorTabs";

            // Der "placeholder", also der Text, der in einem leeren Feld angezeigt werden soll
            settings.Properties.NullText = Api.Text.Format(valueAndAliasControl.NullText);

            // vorbelegen des übergebenen im Datenmodell schon verfügbaren Wertes
            if (!String.IsNullOrWhiteSpace(presetValue))
                settings.PreRender = (sender, arg) =>
                {
                    MVCxTextBox txt = sender as MVCxTextBox;
                    txt.Value = presetValue;
                    txt.DataBind();
                };

            // spezieller Fix für FireFox, damit Text in Feldern selektiert und kopiert werden  
            // können, die keine Eingabe erlauben
            RenderingUtils.FixItemForFirefox(mvcxItem, settings, valueAndAliasControl.CssClass);
            return mvcxItem;
        }

        /// <inheritdoc/>
        public override void ReadMode(FormControlRendererParameterModel parameters)
        {
            FormControlRendererUiParameterModel parametersUi = (FormControlRendererUiParameterModel)parameters;
            FormRenderingContextUi renderingContext = (FormRenderingContextUi)parametersUi.RenderingContext;
            String namePrefix = RenderingUtils.GetUIReadyContextPrefix(renderingContext);
            ValueAndAliasControl valueAndAliasControl = (ValueAndAliasControl)parametersUi.Control;
            String modelFieldName = valueAndAliasControl.OrmFieldName;

            parametersUi.MvcxControls.Add(item =>
            {
                item.NestedExtensionType = FormLayoutNestedExtensionItemType.Label;
                // Adaptive grid layout settings
                item.ColumnSpan = valueAndAliasControl.ColSpan;
                item.RowSpan = valueAndAliasControl.RowSpan;
                foreach (var spanrule in valueAndAliasControl.SpanRules)
                {
                    item.SpanRules.Add(new DevExpress.Web.SpanRule() { BreakpointName = spanrule.BreakpointName, ColumnSpan = spanrule.ColumnSpan, RowSpan = spanrule.RowSpan });
                }

                LabelSettings labelSettings = (LabelSettings)item.NestedExtensionSettings;

                char delimiter = valueAndAliasControl.Delimiter.ToCharArray()[0];
                String fieldContent = (String)parametersUi.BindData.GetPropertyValue(modelFieldName);
                fieldContent = fieldContent?.Split(delimiter)?[0] ?? "";
                labelSettings.Text = fieldContent;

                labelSettings.Name = namePrefix + valueAndAliasControl.ControlInternalName + parametersUi.MvcxControls.Count;
                labelSettings.Width = new Unit(valueAndAliasControl.Width, Api.Enum.EnumValueConverter.GetValueWidthType(valueAndAliasControl.WidthType));
                labelSettings.ControlStyle.CssClass = valueAndAliasControl.CssClass;
                item.HorizontalAlign = FormLayoutHorizontalAlign.Left;
                item.Caption = valueAndAliasControl.Caption?.Translate() ?? String.Empty;
                item.CaptionSettings.Location = Api.Enum.EnumValueConverter.GetLabelPosition(valueAndAliasControl.LabelPosition);

                RenderingUtils.SetHelpText(valueAndAliasControl, item);
                item.Visible = valueAndAliasControl.Visible;

                if (String.IsNullOrWhiteSpace(item.Caption))
                    item.ShowCaption = DevExpress.Utils.DefaultBoolean.False;
            });
        }

        /// <inheritdoc/>
        public override bool Bind(object bindObject, System.Web.Mvc.IValueProvider bindValueProvider, string bindValuePrefix, PropertyDescriptor propertyDescriptor, List<ControlBase> formControls)
        {
            String modelFieldName = propertyDescriptor.Name;
            String bindValueFieldName_Value = bindValuePrefix + modelFieldName + "_Value";
            String bindValueFieldName_Alias = bindValuePrefix + modelFieldName + "_Alias";
            String value = bindValueProvider.GetValue(bindValueFieldName_Value)?.AttemptedValue ?? "";
            String alias = bindValueProvider.GetValue(bindValueFieldName_Alias)?.AttemptedValue ?? "";

            ValueAndAliasControl valueAndAliasControl = (ValueAndAliasControl)formControls.First(ff => (ff is ValueAndAliasControl) && ((ValueAndAliasControl)ff).OrmFieldName == modelFieldName);

            propertyDescriptor.SetValue(bindObject, value + valueAndAliasControl.Delimiter + alias);

            return true;
        }
    }
}