﻿using BA.API;
using BA.Core.CriteriaOperators;
using BA.Core.DataProviders;
using BA.Core.DB;
using BA.Core.Models;
using $safeprojectname$.DB;
using $safeprojectname$.Enums.Extensions;
using DevExpress.Data.Filtering;
using DevExpress.Xpo;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace $safeprojectname$.DataProvider
{
    /// <summary>
    /// Eigener Ansichten Datenprovider für die Maschinen
    /// </summary>
    public class GDPEngines : IGridDataProvider
    {
        /// <inheritdoc/>
        public GridDataProviderPropertiesBase Properties { get; set; }

        /// <inheritdoc/>
        public IQueryable<GridDataProviderRowBase> GetData(GridDataProviderParameter parameter = null)
        {
            if (Properties is GDPEnginesProperties myProperties)
            {
                // LINQ Query erstellen
                Session session = Api.ORM.GetNewSession();
                IQueryable<OrmEngine> query;
                if (myProperties.WithReadPermissions)
                    query = Api.ORM.GetQueryWithReadPermissions<OrmEngine>(session);
                else
                    query = Api.ORM.GetQuery<OrmEngine>(session);

                // Filter erstellen. Einmal  direkt in LINQ und einmal als Formel
                if (!string.IsNullOrEmpty(myProperties.ResponsibleNotContains))
                    // Alle Maschinen die im Betreuerfeld den Propertywert nicht haben.
                    query = query.Where(ff => !ff.Responsible.Contains(myProperties.ResponsibleNotContains));
                else
                {
                    // Alle Maschinen deren Betreuerfeld nicht NULL sind
                    CriteriaOperator criteriaWhere = new OperandProperty(nameof(OrmEngine.Responsible)) != new OperandValue(null);
                    query = query.Where(ff => ff.Fit(criteriaWhere));
                }

                // Formel für die Betreuer Relation
                CriteriaOperator expression = !CriteriaOperatorBuilder.GetSourcesJoin(true, EnumRelationTypeExtension.Responsible, null, onlyRelations: true, aggregator: Aggregate.Exists);
                CriteriaOperator ifpart = new OperandValue("Keine Betreuer Relation");
                CriteriaOperator elsepart = CriteriaOperatorBuilder.GetSourcesJoin(true, EnumRelationTypeExtension.Responsible, null, aggregatedExpression: new OperandProperty(nameof(OrmEngine.EntityTitle)));
                CriteriaOperator coResponsibleRel = new FunctionOperator(FunctionOperatorType.Iif, expression, ifpart, elsepart);

                IQueryable<GridResponsibleRowModel> selectQuery = query.Select(ff => new GridResponsibleRowModel
                {
                    Oid = ff.Oid,
                    RowId = ff.Oid,
                    Name = ff.Name,
                    ResponsibleText = ff.Responsible,
                    ResponsibleRelation = (string)ff.Evaluate(coResponsibleRel),
                });

                return selectQuery;
            }

            throw new Exception("Wrong properties");
        }
    }
}