﻿using BA.API;
using BA.Core.DB;
using BA.Core.Enums;
using BA.Core.Enums.WorkManager;
using BA.Core.MassOperations;
using BA.Core.WorkManager;
using $safeprojectname$.DB;
using $safeprojectname$.Enums.Extensions;
using DevExpress.Xpo;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace $safeprojectname$.Worker
{
    public class AddServiceMassWorker : MassWorkItem, IIgnitableWorkItem
    {
        public Guid TemporaryGuid { get; set; }
        public Guid TaskExecutionId { get; set; }
        public string MessageOnSuccess { get; set; }
        public string MessageOnError { get; set; }
        public string ProgressBarProcessName { get; set; }
        public string ProgressBarProcessDescription { get; set; }
        public IIgnitableAction ClientAction { get; set; }

        public string Remark { get; set; }
        public DateTime Date { get; set; }
        public int MissingRights { get; set; }

        public AddServiceMassWorker() : base()
        {
            ScheduledStartTime = DateTime.UtcNow.AddMinutes(1);
            IsCancellable = true;
            LoggingProcess = EnumLogProcessesExtension.AddService;
            Caption = "281e4152-6d03-40b4-9971-10728e673a00";
            Title = "281e4152-6d03-40b4-9971-10728e673a00";
        }

        protected override IQueryable<OrmBABase> GetQueryable()
        {
            if (NumberOfStarts == 1)
                Logger.AddInfo("61BCA157-96A6-436B-8719-0521B98462B9");
            else
                Logger.AddInfo("227B79DD-31F2-4F6D-A255-94EB8ACA0E89");

            IQueryable<OrmTempSelectedRecords> query = Api.ORM.GetQuery<OrmTempSelectedRecords>(UnitOfWork);
            query = query.Where(ff => ff.TaskExecutionId == TaskExecutionId);

            if (MaxProgress == 0)
            {
                MaxProgress = query.Count();
                Logger.AddInfo("03A9AE68-226F-4F19-91CD-47FA4370230B", MaxProgress);
                throw new Exception(Logger.Translate("160C2DB4-E013-4569-8AB1-A37BDF776707"));
            }
            else
                Logger.AddInfo("E3D32867-83EB-4514-ABEC-C7E596AE192B", CurrentProgress, MaxProgress);

            IQueryable<Guid> oids = query.Select(ff => ff.SelectedRecordOid);
            return Api.ORM.GetQuery<OrmEngine>(UnitOfWork).Where(ff => oids.Contains(ff.Oid));
        }

        protected override void ProcessSingleOrm(OrmBABase ormBABase)
        {
            OrmEngine engine = (OrmEngine)ormBABase;
            Logger.AddInfo("1CBA5EFA-7842-4764-ABB8-05E2FAB5BDE3", engine.Name);

            if (CurrentProgress == 2)
                throw new Exception(Logger.Translate("160C2DB4-E013-4569-8AB1-A37BDF776707"));

            if (engine.IsAllowed(EnumTableOperations.Edit))
            {
                OrmSubEngineServices service = engine.Services.AddNewObject();
                service.SortOrder = engine.Services.Count() - 1;
                service.ServiceDate = Date;
                service.Remark = Remark;
                engine.Save();
            }
            else
            {
                Logger.AddWarning("0DD299DF-C558-44FD-86EC-B3B2ACD47530", engine.Name);
                MissingRights++;
            }

            IQueryable<OrmTempSelectedRecords> query = Api.ORM.GetQuery<OrmTempSelectedRecords>(UnitOfWork);
            query = query.Where(ff => ff.TaskExecutionId == TaskExecutionId && ff.SelectedRecordOid == engine.Oid);
            OrmTempSelectedRecords tempRecord = query.FirstOrDefault();
            if (tempRecord != null)
                tempRecord.Delete();
            else
                Logger.AddWarning("CF6889AB-E880-42C9-8755-BFB15F9F01AF", TaskExecutionId, engine.Oid);
            CurrentProgress++;
        }

        protected override void WorkItemFinished()
        {
            if (State == EnumWorkItemState.Finished)
                if (MissingRights == 0)
                {
                    Api.ClientCommunication.CreateSuccess(Api.User.CurrentUserGuid(), "dd1ceadc-6808-4583-b6b7-dda73188b5a8".Translate(CurrentProgress));
                    Logger.AddInfo("dd1ceadc-6808-4583-b6b7-dda73188b5a8", CurrentProgress);
                }
                else
                {
                    Api.ClientCommunication.CreateError(Api.User.CurrentUserGuid(), "e3f652e7-8019-4444-9e6c-f3c30528e862".Translate(CurrentProgress - MissingRights, MissingRights));
                    Logger.AddInfo("e3f652e7-8019-4444-9e6c-f3c30528e862", CurrentProgress - MissingRights, MissingRights);
                }

            base.WorkItemFinished();
        }
    }
}