﻿using BA.API;
using BA.Core.DB;
using BA.Core.Enums;
using BA.Core.Enums.WorkManager;
using BA.Core.WorkManager;
using $safeprojectname$.DB;
using DevExpress.Xpo;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace $safeprojectname$.Worker
{
    public class AddServiceWorker : WorkItemBase
    {
        public Guid TaskExecutionId { get; set; }

        public string Remark { get; set; }

        public DateTime Date { get; set; }

        public int MissingRights { get; set; }

        public AddServiceWorker() : base()
        {
            ExpectedRunTime = EnumExpectedRunTime.Long;
        }

        protected override void Run()
        {
            IQueryable<OrmTempSelectedRecords> query = Api.ORM.GetQuery<OrmTempSelectedRecords>(UnitOfWork);
            query = query.Where(ff => ff.TaskExecutionId == TaskExecutionId);

            if (MaxProgress == 0)
                MaxProgress = query.Count();

            IQueryable<Guid> oids = query.Select(ff => ff.SelectedRecordOid);
            IQueryable<OrmEngine> engines = Api.ORM.GetQuery<OrmEngine>(UnitOfWork).Where(ff => oids.Contains(ff.Oid));
            foreach (OrmEngine engine in engines)
            {
                if (engine.IsAllowed(EnumTableOperations.Edit))
                {
                    OrmSubEngineServices service = engine.Services.AddNewObject();
                    service.SortOrder = engine.Services.Count() - 1;
                    service.ServiceDate = Date;
                    service.Remark = Remark;
                    engine.Save();
                }
                else
                    MissingRights++;

                query.Where(ff => ff.SelectedRecordOid == engine.Oid).FirstOrDefault()?.Delete();

                if (CurrentProgress == 2 && NumberOfStarts == 1)
                    throw new Exception(Logger.Translate("160C2DB4-E013-4569-8AB1-A37BDF776707"));

                CurrentProgress++;
                Save();
            }
        }

        protected override void WorkItemFinished()
        {
            if (State == EnumWorkItemState.Finished)
                if (MissingRights == 0)                   Api.ClientCommunication.CreateSuccess(Api.User.CurrentUserGuid(), "dd1ceadc-6808-4583-b6b7-dda73188b5a8".Translate(CurrentProgress));
                else
                    Api.ClientCommunication.CreateSuccess(Api.User.CurrentUserGuid(), "e3f652e7-8019-4444-9e6c-f3c30528e862".Translate(CurrentProgress - MissingRights, MissingRights));

            base.WorkItemFinished();
        }
    }
}